	;*************************
;pearls.asm
;*************************
;Joel Seligstein
;Last Modified: July 27, 2006
;*************************
;Pearls is a game popularised by Erich Friedman's paper on the NP-completedness of the puzzle,
;which could be found at http://www.stetson.edu/~efriedma/
;
;The object of the game is to make one continuous, circular band that turns on all black pearls,
;but not their adjacent squares, and does not turn on a white pearl, but does on at least one
;of its adjacents.
;*************************

.nolist
#include "ion.inc"
#include "keys.inc"
.list

#ifdef TI83P
	.org    progstart-2
	.db     $BB,$6D
#else
	.org    progstart
#endif
	ret				
	jr		nc,pearls

.db "Pearls"
saved_text: .db 0,"- saved",0

;*************************
;Variables
;*************************
cursor		= saferam1			 	;1 byte, cursor status (0 = none, 1 = draw, 2 = erase)
pearl_count = cursor		+ 1 	;1 byte, current pearl count
levels		= pearl_count	+ 1		;1 byte, max number of levels
choice		= levels		+ 1		;1 byte, menu choice

;*************************
;Equates	
;*************************
dir_none	.equ $00
dir_up 		.equ $10
dir_right	.equ $20
dir_down	.equ $40
dir_left	.equ $80
dir_ud		.equ $50
dir_lr		.equ $A0
level_count	.equ $14

pearls:
	set		textwrite,(iy+SGRFLAGS)		;Text is written to graphbuffer instead of LCD
  
	ld		hl,saferam1
	ld		(hl),0
	ld		de,saferam1+1
	ld		bc,767
	ldir

	ld		a,level_count
	ld		(levels),a
	

;*************************
;Title Screen
;*************************
;title_text: 	.db "Pearls v1.0",0
;author_text:	.db "By Joel Seligstein",0
;low_text:		.db "Low Score:",0
;new_text:		.db "New Game",0
;continue_text:	.db "Continue",0

title_screen:
	call	clean_buffer
	call	draw_game
	
	ld		hl,2*256+12
	ld		(pencol),hl
	ld		hl,title_text
	bcall(_vputs)
	ld		hl,8*256+2
	ld		(pencol),hl
	ld		hl,author_text
	bcall(_vputs)
	ld		hl,50*256+2
	ld		(pencol),hl
	ld		hl,low_text
	bcall(_vputs)
	ld		hl,50*256+40
	ld		(pencol),hl
	ld		hl,(low_score)
	ld		a,h
	or		l
	jr		z,title_no_low_score
	bcall(_setxxxxop2)
	bcall(_op2toop1)
	ld		a,5
	bcall(_dispop1a)
	jr		z,title_after_score
title_no_low_score:
	ld		hl,no_low_text
	bcall(_vputs)
title_after_score:
	xor		a
	ld		(choice),a
title_menu:
	ld		a,(choice)
	or		a
	jr		nz,title_menu_not_choice_0
	set		textinverse,(iy+textflags)	;Inverse text
title_menu_not_choice_0:
	ld		hl,20*256+13
	ld		(pencol),hl
	ld		hl,new_text
	bcall(_vputs)
	res		textinverse,(iy+textflags)	;Inverse text
	ld		a,(saved_text)
	or		a
	jr		z,title_menu_no_continue
	ld		a,(choice)
	dec		a
	jr		nz,title_menu_not_choice_1
	set		textinverse,(iy+textflags)	;Inverse text
title_menu_not_choice_1:
	ld		hl,30*256+15
	ld		(pencol),hl
	ld		hl,continue_text
	bcall(_vputs)
	res		textinverse,(iy+textflags)	;Inverse text
title_menu_no_continue:
	call	ionfastcopy
title_menu_wait_for_key
	bcall(_getcsc)	
	cp		skDown
	jr		z,toggle_choice
	cp		skUp
	jr		z,toggle_choice
	cp		skClear
	ret		z
	cp		skMode
	ret		z
	cp		skEnter
	jr		nz,title_menu_wait_for_key	
title_menu_do_choice:
	ld		a,(choice)
	or		a
	jp		nz,display_and_go
new_game:
	xor		a
	ld		(level),a
	ld		(moves),a
	ld		(moves+1),a
	jp		load_level


toggle_choice:
	ld		a,(saved_text)
	or		a
	jr		z,title_menu
	ld		a,(choice)
	xor		1
	ld		(choice),a
	jr		title_menu

	
	
	
game_loop:
	ld		b,10
	ei
halt_loop:
	halt
	djnz	halt_loop
	di
	
game_loop_skip_halt:
	xor		a
	ld		(cursor),a
	
	ld		a,0ffh
	out		(1),a
	ld		a,Group7
	out		(1),a
	in		a,(1)
	
	cp		k2nd
	jr		nz,no_2nd
	ld		a,1
	ld		(cursor),a
	jr		game_loop_after_alpha
	
no_2nd:
	cp		KDel
	jr		nz,game_loop_no_del

save_game:
	ld		hl,saved_text
	ld		(hl),' '
	ret
dont_save_game:
	ld		hl,saved_text
	ld		(hl),0
	ret

game_loop_no_del:
	cp		kMode
	jr		nz,game_loop_no_mode
	jp		load_level

game_loop_no_mode:
	ld		a,0ffh
	out		(1),a
	ld		a,Group2
	out		(1),a
	in		a,(1)
	
	cp		KClear
	jr		z,dont_save_game
	
	ld		a,0ffh
	out		(1),a
	ld		a,Group6
	out		(1),a
	in		a,(1)
	
	cp		kAlpha
	jr		nz,game_loop_after_alpha
	ld		a,2
	ld		(cursor),a


game_loop_after_alpha:	
	ld		a,0ffh
	out		(1),a
	ld		a,Group1
	out		(1),a
	in		a,(1)
	
	cp		KLeft
	jr		z,move_left
	
	cp		KRight
	jr		z,move_right
	
	cp		KDown
	jr		z,move_down
	
	cp		KUp
	jr		nz,game_loop_skip_halt
	
move_up:								;hl = old, de = new, bc = dirs
	ld		a,(cur_y)					;if( cur_y >= 0 ) return
	or		a
	jr		z,game_loop
	
	ld		h,a							;save old y
	dec		a
	ld		(cur_y),a					;save new y
	ld		d,a
	
	ld		a,(cur_x)					;get old/new x (same)
	ld		e,a
	ld		l,a
	
	ld		b,dir_up					;load dirs
	ld		c,dir_down
	
	jr		update_line

move_down:								;hl = old, de = new, bc = dirs
	ld		a,(cur_y)
	ld		c,a
	ld		a,(height)					;subtract: height - current_y, decrease one and check if 0 (cur_y < height-1)
	sub		c
	dec		a
	jp		z,game_loop					;if( cur_y == height - 1 ) return
	
	ld		a,c
	ld		h,a							;save old y
	inc		a
	ld		(cur_y),a					;save new y
	ld		d,a
	
	ld		a,(cur_x)					;get old/new x (same)
	ld		e,a
	ld		l,a
	
	ld		b,dir_down					;load dirs
	ld		c,dir_up
	
	jr		update_line
	
move_left:								;hl = old, de = new, bc = dirs
	ld		a,(cur_x)					;if( x == 0 ) return;
	or		a
	jp		z,game_loop
	
	ld		l,a							;save old x
	dec		a
	ld		(cur_x),a					;save new x
	ld		e,a
	
	ld		a,(cur_y)					;get old/new y (same)
	ld		h,a
	ld		d,a
	
	ld		b,dir_left					;load dirs
	ld		c,dir_right
	
	jr		update_line

move_right:								;hl = old, de = new, bc = dirs
	ld		a,(cur_x)					;if( cur_y == height - 1 ) return
	ld		c,a
	ld		a,(width)
	sub		c
	dec		a
	jp		z,game_loop
	
	ld		a,c
	ld		l,a							;save old x
	inc		a
	ld		(cur_x),a					;save new x
	ld		e,a
	
	ld		a,(cur_y)					;get old/new y (same)
	ld		h,a
	ld		d,a
	
	ld		b,dir_right
	ld		c,dir_left
	
	;jr		update_line					;inline

;*************************
;Update Line (inline)
;*************************
;IN      : h = old y, l = old x
;          d = new y, e = new x
;          b = old direction bit, c = new direction bit
;OUT     : Values set according to (cursor) and direction information
;DESTROYS:

update_line:
	ld		a,(cursor)
	dec		a
	jr		z,draw_line
	dec		a
	jp		nz,check_and_draw

erase_line:							;hl = old, de = new, bc = dirs
	push	de						; new_xy
	push	hl						; old_xy | new_xy
	push	bc						; dirs | old_xy | new_xy

	ld		b,h						;bc = old_xy
	ld		c,l	
	call	get_value_from_tilemap
	pop		bc						;bc = dirs <-- old_xy | new_xy
	ld		d,a						;stack = old_xy | new_xy, bc = dirs, d = value
	
	ld		a,b						;compliment the direction, AND it, reset the value
	cpl
	and		d
	pop		hl						; hl = old_xy <-- new_xy
	push	bc						; dirs | new_xy
	ld		b,h
	ld		c,l
	call	set_value_in_tilemap
	
	pop		de						; de = dirs <-- new_xy
	pop		bc						; bc = new_xy <--
	push	bc						; new_xy
	push	de						; dirs | new_xy
	call	get_value_from_tilemap
	ld		d,a
	
	pop		bc						; bc = dirs <-- new_xy
	ld		a,c						;compliment the direction, AND it, reset the value
	cpl
	and		d
	pop		bc						; bc = new_xy
	call	set_value_in_tilemap
	
	jp		check_and_draw			;done
	
	
draw_line:							;hl = old, de = new, bc = dirs
	push	de						; new_xy
	push	hl						; old_xy | new_xy
	push	bc						; dirs | old_xy | new_xy
	
	ld		b,h						;bc = old_xy
	ld		c,l
	call	get_value_from_tilemap
	
	pop		bc						; bc = dirs <- old_xy | new_xy
	or		b						;set direction bit
	pop		de						; de = old_xy <-- new_xy
	push	bc						; dirs | new_xy
	ld		b,d						; bc = old_xy
	ld		c,e
	call	set_value_in_tilemap
	
	pop		de						; de = dirs <-- new_xy
	pop		bc						; bc = new_xy <--
	push	bc						; new_xy
	push	de						; dirs | new_xy
	call	get_value_from_tilemap
	
	pop		de						; de = dirs <-- new_xy
	pop		bc						; bc = new_xy <--
	or		e						;set direction bit
	call	set_value_in_tilemap
	
	;jp		game_loop				;done

;*************************
;Check And Draw
;*************************
check_and_draw:
	ld		hl,(moves)
	inc		hl
	ld		(moves),hl
	
	call	check_level
	or		a
	jr		nz,next_level
	
display_and_go:
	call	draw_level
	call	ionFastCopy
	
	jp		game_loop

next_level:
	ld		hl,level
	ld		a,(levels)
	sub		(hl)
	dec		a
	jr		z,game_over
	inc		(hl)
	
load_level:
	xor		a
	ld		(cur_x),a
	ld		(cur_y),a
	ld		a,(level)
	call	find_level
	call	decompress_level
	
	jr		display_and_go
	
game_over:
	xor		a
	ld		(saved_text),a
	
	call	draw_level
	
	set		textinverse,(iy+textflags)	;Inverse text
  
	ld		hl,(low_score)
	ld		de,(moves)
	ld		a,h
	or		l
	jr		z,new_low_score
	
	or		a
	sbc		hl,de
	jr		z,no_low_score
	jp		m,no_low_score
new_low_score:
	ld		(low_score),de
	ld		hl,35*256+5
	ld		(pencol),hl
	ld		hl,new_score_text
	bcall(_vputs)
no_low_score:
	ld		hl,25*256+13
	ld		(pencol),hl
	ld		hl,game_over_text
	bcall(_vputs)
	
	res		textinverse,(iy+textflags)	;Inverse text
  	
  	call	ionfastcopy

wait_for_enter:
	bcall(_getcsc)
	cp		skEnter
	jr		nz,wait_for_enter
	res		textwrite,(iy+SGRFLAGS)		;Text is written to graphbuffer instead of LCD
	ret

;*************************
;Check Level
;*************************
;IN      : (width), (height), and cur_level set
;OUT     : a = 1, level complete, a = 0, not complete
;DESTORYS: hl,a,bc,de
check_level:
	call	get_level_map_size			;find the map size to traverse and find first line
	ld		hl,cur_level
	ld		b,a
check_level_find_line:
	ld		a,(hl)						;check if this square has any lines
	and		$F0
	jr		nz,check_level_start_trace	;found one
	inc		hl							;keep trying
	djnz	check_level_find_line
	xor		a							;no lines, return not complete
	ret
check_level_start_trace:
	push	hl							; start_pt     save starting point
	
	xor		a							;reset pearl counter
	ld		(pearl_count),a
	
	ld		a,(hl)						;find and remove a last direction, then jump to skipping the loop comparison
	and		dir_up
	jr		z,check_level_no_up
	ld		c,dir_up
	jr		check_level_skip_first
check_level_no_up:
	ld		a,(hl)
	and		dir_down
	jr		z,check_level_no_down
	ld		c,dir_down
	jr		check_level_skip_first
check_level_no_down:
	ld		a,(hl)
	and		dir_left
	jr		z,check_level_no_left
	ld		c,dir_left
	jr		check_level_skip_first
check_level_no_left:
	ld		c,dir_right
	jr		check_level_skip_first

check_level_next:						;check the next piece of the level	
	pop		de							;compare new position to old, if they're the same, we have a loop
	push	hl							;save current position
	or		a
	sbc		hl,de
	pop		hl							;re-get position
	jr		z,check_level_verify
	push	de							;resave position
check_level_skip_first:
	ld		a,(hl)						;check if this is a pearl
	and		$0F
	jr		z,check_level_no_pearl
	push	hl							;validate and count the pearl
	push	bc
	call	verify_pearl
	or		a
	jr		nz,check_level_valid_pearl
	pop		bc
	pop		hl
	jr		check_level_invalid

check_level_valid_pearl:
	ld		hl,pearl_count
	inc		(hl)
	pop		bc
	pop		hl
	
check_level_no_pearl:
	ld		a,c							;get the new byte, take out the last direction
	cpl
	and		(hl)
	and		$F0
	jr		z,check_level_invalid		;at this point, we've found no new directions, dead end
	
check_level_continue:
	cp		dir_down					;check if other dir is down
	jr		z,check_level_down
	cp		dir_left					;check if other dir is left
	jr		z,check_level_left
	cp		dir_right					;check if other dir is right
	jr		z,check_level_right
	cp		dir_up						;check if other dir is up
	jr		nz,check_level_invalid		;at this point, more than one direction = multiple = incomplete
	
check_level_up:							;update for the next iteration based on direction
	ld		a,(width)
	ld		d,0
	ld		e,a
	or		a
	sbc		hl,de
	ld		c,dir_down
	jr		check_level_next

check_level_down:
	ld		a,(width)
	ld		d,0
	ld		e,a
	add		hl,de
	ld		c,dir_up
	jr		check_level_next
	
check_level_left:
	dec		hl
	ld		c,dir_right
	jr		check_level_next
	
check_level_right:
	inc		hl
	ld		c,dir_left
	jr		check_level_next
	
check_level_verify:						;compare the verified pearl count to the total
	call	get_pearl_count				;level pearl count to make sure we went through
	ld		d,a							;every pearl
	ld		a,(pearl_count)
	sub		d
	jr		nz,check_level_invalid_no_pop ;not the same, no winner
check_level_complete:
	inc		a
	ret
check_level_invalid:
	pop		hl
check_level_invalid_no_pop:
	xor		a
	ret

;*************************
;Verify Pearl
;*************************
;IN      : (width), (height), and cur_level set
;          hl points to a pearl within cur_level
;OUT     : a = 0, pearl invalid, a = 1, pearl valid
;DESTROYS: af
verify_pearl:
	push	bc
	push	de
	push	hl
	ld		a,(hl)						;find out which pearl
	and		$0F
	dec		a
	jr		nz,verify_black_pearl
verify_white_pearl:
	ld		a,(hl)						;check if either UD or LR
	and		dir_lr
	jr		nz,verify_white_pearl_lr
	ld		a,(hl)
	and		dir_ud
	jr		nz,verify_white_pearl_ud
verify_pearl_invalid:
	xor		a							;neither, invalid.
	jp		verify_pearl_return

verify_white_pearl_lr:
	inc		hl
	ld		a,(hl)
	and		$F0
	cp		dir_lr
	jr		nz,verify_pearl_valid
	dec		hl
	dec		hl
	ld		a,(hl)
	and		$F0
	cp		dir_lr
	jr		nz,verify_pearl_valid
	jr		verify_pearl_invalid
	
verify_white_pearl_ud:
	push	hl
	ld		a,(width)
	ld		d,0
	ld		e,a
	add		hl,de
	ld		a,(hl)
	and		$F0
	cp		dir_ud
	pop		hl
	jr		nz,verify_pearl_valid
	or		a
	sbc		hl,de
	ld		a,(hl)
	and		$F0
	cp		dir_ud
	jr		nz,verify_pearl_valid
	jr		verify_pearl_invalid
	
verify_black_pearl:
	ld		a,(hl)						;check if a line is straight through
	and		$F0
	cp		dir_ud
	jr		z,verify_pearl_invalid
	ld		a,(hl)
	and		$F0
	cp		dir_lr
	jr		z,verify_pearl_invalid
	
	ld		a,(hl)
	and		dir_up						;check each direction
	jr		z,verify_black_pearl_right
verify_black_pearl_up:
	ld		a,(width)
	ld		d,0
	ld		e,a
	or		a
	sbc		hl,de
	ld		a,(hl)
	and		$F0
	cp		dir_ud
	jr		nz,verify_pearl_invalid
	add		hl,de
	
verify_black_pearl_right:
	ld		a,(hl)
	and		dir_right
	jr		z,verify_black_pearl_down
	inc		hl
	ld		a,(hl)
	and		$F0
	cp		dir_lr
	jr		nz,verify_pearl_invalid
	dec		hl
	
verify_black_pearl_down:
	ld		a,(hl)
	and		dir_down
	jr		z,verify_black_pearl_left
	ld		a,(width)
	ld		d,0
	ld		e,a
	add		hl,de
	ld		a,(hl)
	and		$F0
	cp		dir_ud
	jr		nz,verify_pearl_invalid
	or		a
	sbc		hl,de
	
verify_black_pearl_left:
	ld		a,(hl)
	and		dir_left
	jr		z,verify_pearl_valid
	dec		hl
	ld		a,(hl)
	and		$F0
	cp		dir_lr
	jp		nz,verify_pearl_invalid
	
verify_pearl_valid:
	ld		a,1
verify_pearl_return:
	pop		hl
	pop		de
	pop		bc
	ret

	
;*************************
;Get Level Map Size
;*************************
;IN      : (width), (height), and cur_level set
;OUT     : a = (width)*(height)
;DESTROYS: af,bc
get_level_map_size:
	ld		a,(width)
	ld		b,a
	ld		a,(height)
	ld		c,a
	xor		a
get_level_map_size_multiply:
	add		a,c
	djnz	get_level_map_size_multiply
	ret

	

;*************************
;Count Pearls
;*************************
;IN      : (width), (height), and cur_level set
;OUT     : a = total pearl count
;          d = white pearl count
;          e = black pearl count
;DESTORYS: hl,a,bc,de
get_pearl_count:
	call	get_level_map_size			;find the map size to traverse
	ld		de,0
	ld		hl,cur_level
	ld		b,a
get_pearl_count_traverse:
	ld		a,(hl)						;check if this square is a pearl
	and		$0F
	dec		a
	jr		nz,get_pearl_count_not_white
	inc		d
	jr		get_pearl_count_next
get_pearl_count_not_white:
	dec		a
	jr		nz,get_pearl_count_next
	inc		e
get_pearl_count_next:
	inc		hl
	djnz	get_pearl_count_traverse
	ld		a,d
	add		a,e
	ret

;*************************
;Get Location Of Value for (x,y)
;*************************
;IN      : (width), (height), and cur_level set
;          b = y, c = x
;OUT     : hl = location of (x,y) in tilemap
;DESTROYS: de,a,b
get_location:
	ld		hl,cur_level
	ld		a,(width)
	ld		e,a
	ld		a,b
	or		a
	jr		z,get_loc_skip_rows
	xor		a
get_loc_increment_row:
	add		a,e
	djnz	get_loc_increment_row
get_loc_skip_rows:
	add		a,c
	ld		d,0
	ld		e,a
	add		hl,de
	ret
	
;*************************
;Get Value From Tilemap
;*************************
;IN      : (width), (height), and cur_level set
;          b = y, c = x
;OUT     : a = value
;DESTROYS: hl,de,a,b
get_value_from_tilemap:
    call	get_location
    ld		a,(hl)
    ret

;*************************
;Set Value In Tilemap
;*************************
;IN      : (width), (height), and cur_level set
;          b = y, c = x
;          a = value
;OUT     : value set
;DESTROYS: hl,de,a,bc
set_value_in_tilemap:
    push	af
    call	get_location
    pop		af
    ld		(hl),a
    ret

;*************************
;Clean Buffer
;*************************
;IN      : none
;OUT     : plotsscreen cleared
;DESTROYS: all
clean_buffer:
	ld		hl,plotsscreen
	ld		(hl),0
	ld		de,plotsscreen+1
	ld		bc,767
	ldir
	ret
	
;*************************
;Draw 6x6 Tilemap
;*************************
;IN      : hl = tilemap location (not anymore, now its cur_level)
;OUT     : tilemap drawn to screen via (width),(height)
;DESTROYS: all (hl is tilemap end+1)
;NOTES   : This is a bit slow using ionPutSprite, but I am much too lazy to write bit shifting
draw_level:
	call	clean_buffer
	call	draw_game
	
	ld		a,(width)					;draw level bounding lines
	add		a,a							;2a
	ld		d,a
	add		a,a							;4a
	add		a,d							;6a
	inc		a
	ld		d,a
	
	ld		a,(height)
	add		a,a
	ld		e,a
	add		a,a
	add		a,e
	inc		a
	ld		e,a
	
	push	de
	ld		b,e
	inc		b
	ld		e,0
	call	V_Line
	
	pop		de
	ld		b,d
	ld		d,0
	call	H_Line
	
	ld		hl,cur_level
	ld		de,1					;start at (1,1)
	ld		bc,1
draw_6x6_inc_col:
	push	bc
	ld		b,0						;load bc with current title
	ld		a,(hl)					;drop the line sprites for now
	and		$0F
	add		a,a						;2a
	ld		c,a
	add		a,a						;4a
	add		a,c						;6a
	ld		c,a
	ld		ix,map_sprites			;add it to sprite location
	add		ix,bc
	pop		bc
	push	hl						;save tile location and coordinates
	push	de
	push	bc
	ld		b,6						;set height
	ld		l,e
	ld		a,c
	call	ionPutSprite			;draw the sprites
	pop		bc						;draw the lines now
	pop		de
	pop		hl
	push	bc
	ld		b,0
	ld		a,(hl)
	srl		a
	srl		a
	srl		a
	srl		a
	and		$0F
	add		a,a						;2a
	ld		c,a
	add		a,a						;4a
	add		a,c						;6a
	ld		c,a
	ld		ix,line_sprites
	add		ix,bc
	pop		bc
	push	hl
	push	de
	push	bc
	ld		b,6
	ld		l,e
	ld		a,c
	call	ionPutSprite
	pop		bc
	pop		de						;get tile location and coordinates
	pop		hl
	inc		hl						;next tile
	
	ld		a,c						;increase the x coordinate
	add		a,6
	ld		c,a
	inc		b
	ld		a,(width)
	sub		b
	jr		nz,draw_6x6_inc_col		;nope, draw next x
	
draw_6x6_inc_row:
	ld		a,e						;increase the y coordinate
	add		a,6
	ld		e,a
	inc		d
	ld		a,(height)
	sub		d
	jr		z,draw_level_cursor
	ld		bc,1
	jr		draw_6x6_inc_col
	
draw_level_cursor:
	ld		a,(cur_y)
	add		a,a
	ld		c,a
	add		a,a
	add		a,c
	inc		a
	ld		l,a
	ld		a,(cur_x)
	add		a,a
	ld		c,a
	add		a,a
	add		a,c
	inc		a
	ld		b,6
	ld		ix,cursor_sprite
	call	ionPutSprite
	ret
	
;*************************
;Draw Game
;*************************
;IN      : none
;OUT     : lines, logo, etc drawn to screen
;DESTROYS: all
draw_game:
	ld		de,0
	ld		b,96
	call	H_Line
	
	ld		de,0*256+61
	ld		b,96
	call	H_Line
	
	ld		de,0
	ld		b,62
	call	V_Line
	
	ld		de,61*256+0
	ld		b,62
	call	V_Line
	
	ld		de,95*256+0
	ld		b,62
	call	V_Line
	
	ld		hl,23*256+63
	ld		(pencol),hl
	ld		hl,level_text
	bcall(_vputs)	
	
	ld		hl,29*256+63
	ld		(pencol),hl
	ld		hl,(level)
	ld		h,0
	inc		l
	bcall(_setxxxxop2)
	bcall(_op2toop1)
	ld		a,5
	bcall(_dispop1a)
	
	ld		hl,40*256+63
	ld		(pencol),hl
	ld		hl,moves_text
	bcall(_vputs)
	
	ld		hl,46*256+63
	ld		(pencol),hl
	ld		hl,(moves)
	bcall(_setxxxxop2)
	bcall(_op2toop1)
	ld		a,5
	bcall(_dispop1a)
	
	ld		ix,logo
	ld		bc,12*256+4
	ld		l,2
	ld		a,63
	call	ionLargeSprite
	
	ret

;*************************
;Find level
;*************************
;IN      : a = level to find
;OUT     : hl = level data
;DESTROYS: b,de,hl
find_level:
	ld		b,a						;save a counter
	ld		hl,maps					;preload pointer
	or		a						;if level 0, return now
	ret		z
find_level_next:	
	ld		a,(hl)					;find the height (low nibble)
	and		$0F
	ld		d,a
	
	ld		a,(hl)					;find the width (high nibble)
	srl		a
	srl		a
	srl		a
	srl		a
	ld		e,a
	
	xor		a						;initialise to 0, then multiply by repetitive addition
find_level_multiply:
	add		a,e
	dec		d
	jr		nz,find_level_multiply
	
	ld		e,a						;need to figure out if there is a remainder to tack on for one extra byte
	and		$07						;if any of last 3 bits are set, answer is yes
	jr		z,find_level_divide
	ld		a,e						;yes add it (by setting the 2nd bit)
	or		$04
	ld		e,a

find_level_divide:
	srl		e						;divide by 4 (map size * 2pixels per square / 8 bits per byte)
	srl		e
	
	inc		hl						;move to the first byte of map data
	
	ld		d,0						;add this offset to get the next level
	add		hl,de
	
	djnz	find_level_next			;repeat until level is found
	ret	

;*************************
;Decompress level
;*************************
;IN      : hl = level to decompress
;OUT     : (width) = width of level
;          (height) = height of level
;          cur_level = beginning of level data
;          title = pointer to title
;DESTROYS: all
decompress_level:
	push	hl						;clear out the level data
	ld		hl,cur_level
	ld		(hl),0
	ld		de,cur_level+1
	ld		bc,10*10
	ldir
	pop		hl
	
	ld		a,(hl)					;load the width
	srl		a
	srl		a
	srl		a
	srl		a
	and		$0F
	ld		(width),a
	ld		d,a
	
	ld		a,(hl)					;load the height
	and		$0F
	ld		(height),a
	ld		e,a

	xor		a						;initialise to 0, then multiply by repetitive addition
decompress_level_multiply:
	add		a,e
	dec		d
	jr		nz,decompress_level_multiply
	
	ld		b,a						;need to figure out if there is a remainder to tack on for one extra byte
	and		$07						;if any of last 3 bits are set, answer is yes
	jr		z,decompress_level_divide
	ld		a,b						;yes add it (by setting the 2nd bit)
	or		$04
	ld		b,a

decompress_level_divide:
	srl		b						;divide by 4 (map size * 2pixels per square / 8 bits per byte)
	srl		b

	inc		hl						;move to the first byte of map data
	ld		de,cur_level			;place to save level data to
decompress_level_decompress:
	ld		a,(hl)					;save byte 1
	;rl		a						;shift right 6
	;rl		a
	srl		a
	srl		a
	srl		a
	srl		a
	srl		a
	srl		a
	and		$03
	ld		(de),a
	inc		de
	
	ld		a,(hl)					;save byte 2
	srl		a						;shift right 4 (todo: way to flip nibbles?)
	srl		a
	srl		a
	srl		a
	and		$03
	ld		(de),a
	inc		de
	
	ld		a,(hl)					;save byte 3
	srl		a						;shift right 2
	srl		a
	and		$03
	ld		(de),a
	inc		de
	
	ld		a,(hl)					;save byte 4
	and		$03
	ld		(de),a
	inc		de
	inc		hl
	
	djnz	decompress_level_decompress		;repeat
	ret


;=======================================;
; Horizontal and Verticle Line Routines ;
; By Jason Kovacs & The TCPA - 10/11/99 ;
; Modified for Basic Tutor  -  03/17/01 ;
; Used by Pearls             - 08/01/06 ;
;=======================================;
; For H_Line and V_Line:
;
; Input:  B = Length of Line (Number of Pixels)
;         C = Color of Line (0-White, 1-Black, 2-XORed)
;         C is no longer used, lines are always Black
;         D = X Coordinate Start of the Line
;         E = Y Coordinate Start of the Line
;
; Output: Lines are Drawn to the Graph Buffer, and the Starting
;    Byte and Pixel Mask are Automatically determined according
;    to the Input of the Coordinates in DE.
;
; Registers Affected:  All Registers are Destroyed ;;;Except BC

V_Line:
	push	bc
	ld		a,d
	call	Getpix
	pop		bc
	ld		c,a
	ld		de,12
V_Black_Line
	ld		a,(hl)
	or		c
	ld		(hl),a
	add		hl,de
	djnz	V_Black_Line
    ret
	
H_Line:
	push	bc
	ld		a,d
	call	Getpix
	pop		bc
	ld		c,a
H_Black_Line:
	ld		a,(hl)
	or		c
	ld		(hl),a
	rr		c
	jr		nc,Cont_H_Black_Line
	inc		hl
	rr		c
Cont_H_Black_Line:
	djnz	H_Black_Line
	ret

Getpix:
	ld		d,0
	ld		h,d
	ld		l,e
	add		hl,de
	add		hl,de
	add		hl,hl
	add		hl,hl
	ld		de,plotsscreen
	add		hl,de
	
Getbit:
	ld		b,0
	ld		c,a
	and		$07
	srl		c
	srl		c
	srl		c
	add		hl,bc
	ld		b,a
	inc		b
	ld		a,$01
GBLoop:
	rrca
	djnz	GBLoop
	ret
	
	
;*************************
;Map Sprites
;*************************
map_sprites:
blank:
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
white_pearl:
.db %01111000
.db %10000100
.db %10000100
.db %10000100
.db %10000100
.db %01111000
black_pearl:
.db %01111000
.db %11111100
.db %11111100
.db %11111100
.db %11111100
.db %01111000

line_sprites:
line_none:
.db	%00000000
.db %00000000
.db	%00000000
.db %00000000
.db	%00000000
.db %00000000
line_u:
.db	%00110000
.db %00110000
.db	%00110000
.db %00000000
.db	%00000000
.db %00000000
line_r:
.db	%00000000
.db %00000000
.db	%00011100
.db %00011100
.db	%00000000
.db %00000000
line_ur:
.db	%00110000
.db %00110000
.db	%00111100
.db %00011100
.db	%00000000
.db %00000000
line_d:
.db	%00000000
.db %00000000
.db	%00000000
.db %00110000
.db	%00110000
.db %00110000
line_ud:
.db	%00110000
.db %00110000
.db	%00110000
.db %00110000
.db	%00110000
.db %00110000
line_dr:
.db	%00000000
.db %00000000
.db	%00011100
.db %00111100
.db	%00110000
.db %00110000
line_udr:
.db	%00110000
.db %00110000
.db	%00111100
.db %00111100
.db	%00110000
.db %00110000
line_l:
.db	%00000000
.db %00000000
.db	%11100000
.db %11100000
.db	%00000000
.db %00000000
line_ul:
.db	%00110000
.db %00110000
.db	%11110000
.db %11100000
.db	%00000000
.db %00000000
line_rl:
.db	%00000000
.db %00000000
.db	%11111100
.db %11111100
.db	%00000000
.db %00000000
line_url:
.db	%00110000
.db %00110000
.db	%11111100
.db %11111100
.db	%00000000
.db %00000000
line_dl:
.db	%00000000
.db %00000000
.db	%11100000
.db %11110000
.db	%00110000
.db %00110000
line_udl:
.db	%00110000
.db %00110000
.db	%11110000
.db %11110000
.db	%00110000
.db %00110000
line_rdl:
.db	%00000000
.db %00000000
.db	%11111100
.db %11111100
.db	%00110000
.db %00110000
line_urdl:
.db	%00110000
.db %00110000
.db	%11111100
.db %11111100
.db	%00110000
.db %00110000

cursor_sprite:
.db $FC,$FC,$FC,$FC,$FC,$FC

;*************************
;Continue Data
;*************************
low_score:	.dw 0
level:		.db 0
moves:		.dw 0
cur_x:		.db 0
cur_y:		.db 0
width:		.db 0
height:		.db 0
cur_level	.ds 101

;*************************
;Text Data
;*************************
title_text: 	.db "Pearls v1.0",0
author_text:	.db "By Joel Seligstein",0
low_text:		.db "Low Score:",0
new_text:		.db " New Game",0
continue_text:	.db " Continue",0
level_text:		.db "LEVEL:",0
moves_text:		.db "MOVES:",0
game_over_text: .db " Great job!",0
new_score_text: .db " New Low Score!",0
no_low_text:	.db "None",0

;*************************
;Level Data
;*************************
;maps:
#include levels.inc

;*************************
;Logo Data
;*************************
logo:
.db %00001110,%01110010,%00100111,%00001000
.db %00010001,%10010111,%00111101,%00111000
.db %00101000,%11101101,%00100101,%01001000
.db %00100001,%11011101,%10101101,%10110000
.db %00110011,%11111010,%01101101,%11010000
.db %00111111,%10011011,%01101100,%11101000
.db %00101110,%01111011,%01111111,%10010000
.db %00101000,%00001100,%11010000,%11100000
.db %00101000,%00001000,%01000000,%10000000
.db %00101000,%00000000,%00000000,%00000000
.db %00110000,%00000000,%00000000,%00000000
.db %00100000,%00000000,%00000000,%00000000

.end